@@ -1,14 +1,14 @@
 mod anthropic;
+mod cloud;
 #[cfg(test)]
 mod fake;
 mod open_ai;
-mod zed;
 
 pub use anthropic::*;
+pub use cloud::*;
 #[cfg(test)]
 pub use fake::*;
 pub use open_ai::*;
-pub use zed::*;
 
 use crate::{
     assistant_settings::{AssistantProvider, AssistantSettings},
@@ -25,31 +25,26 @@ use std::time::Duration;
 pub fn init(client: Arc<Client>, cx: &mut AppContext) {
     let mut settings_version = 0;
     let provider = match &AssistantSettings::get_global(cx).provider {
-        AssistantProvider::ZedDotDev { default_model } => {
-            CompletionProvider::ZedDotDev(ZedDotDevCompletionProvider::new(
-                default_model.clone(),
-                client.clone(),
-                settings_version,
-                cx,
-            ))
-        }
+        AssistantProvider::ZedDotDev { model } => CompletionProvider::Cloud(
+            CloudCompletionProvider::new(model.clone(), client.clone(), settings_version, cx),
+        ),
         AssistantProvider::OpenAi {
-            default_model,
+            model,
             api_url,
             low_speed_timeout_in_seconds,
         } => CompletionProvider::OpenAi(OpenAiCompletionProvider::new(
-            default_model.clone(),
+            model.clone(),
             api_url.clone(),
             client.http_client(),
             low_speed_timeout_in_seconds.map(Duration::from_secs),
             settings_version,
         )),
         AssistantProvider::Anthropic {
-            default_model,
+            model,
             api_url,
             low_speed_timeout_in_seconds,
         } => CompletionProvider::Anthropic(AnthropicCompletionProvider::new(
-            default_model.clone(),
+            model.clone(),
             api_url.clone(),
             client.http_client(),
             low_speed_timeout_in_seconds.map(Duration::from_secs),
@@ -65,13 +60,13 @@ pub fn init(client: Arc<Client>, cx: &mut AppContext) {
                 (
                     CompletionProvider::OpenAi(provider),
                     AssistantProvider::OpenAi {
-                        default_model,
+                        model,
                         api_url,
                         low_speed_timeout_in_seconds,
                     },
                 ) => {
                     provider.update(
-                        default_model.clone(),
+                        model.clone(),
                         api_url.clone(),
                         low_speed_timeout_in_seconds.map(Duration::from_secs),
                         settings_version,
@@ -80,27 +75,24 @@ pub fn init(client: Arc<Client>, cx: &mut AppContext) {
                 (
                     CompletionProvider::Anthropic(provider),
                     AssistantProvider::Anthropic {
-                        default_model,
+                        model,
                         api_url,
                         low_speed_timeout_in_seconds,
                     },
                 ) => {
                     provider.update(
-                        default_model.clone(),
+                        model.clone(),
                         api_url.clone(),
                         low_speed_timeout_in_seconds.map(Duration::from_secs),
                         settings_version,
                     );
                 }
-                (
-                    CompletionProvider::ZedDotDev(provider),
-                    AssistantProvider::ZedDotDev { default_model },
-                ) => {
-                    provider.update(default_model.clone(), settings_version);
+                (CompletionProvider::Cloud(provider), AssistantProvider::ZedDotDev { model }) => {
+                    provider.update(model.clone(), settings_version);
                 }
-                (_, AssistantProvider::ZedDotDev { default_model }) => {
-                    *provider = CompletionProvider::ZedDotDev(ZedDotDevCompletionProvider::new(
-                        default_model.clone(),
+                (_, AssistantProvider::ZedDotDev { model }) => {
+                    *provider = CompletionProvider::Cloud(CloudCompletionProvider::new(
+                        model.clone(),
                         client.clone(),
                         settings_version,
                         cx,
@@ -109,13 +101,13 @@ pub fn init(client: Arc<Client>, cx: &mut AppContext) {
                 (
                     _,
                     AssistantProvider::OpenAi {
-                        default_model,
+                        model,
                         api_url,
                         low_speed_timeout_in_seconds,
                     },
                 ) => {
                     *provider = CompletionProvider::OpenAi(OpenAiCompletionProvider::new(
-                        default_model.clone(),
+                        model.clone(),
                         api_url.clone(),
                         client.http_client(),
                         low_speed_timeout_in_seconds.map(Duration::from_secs),
@@ -125,13 +117,13 @@ pub fn init(client: Arc<Client>, cx: &mut AppContext) {
                 (
                     _,
                     AssistantProvider::Anthropic {
-                        default_model,
+                        model,
                         api_url,
                         low_speed_timeout_in_seconds,
                     },
                 ) => {
                     *provider = CompletionProvider::Anthropic(AnthropicCompletionProvider::new(
-                        default_model.clone(),
+                        model.clone(),
                         api_url.clone(),
                         client.http_client(),
                         low_speed_timeout_in_seconds.map(Duration::from_secs),
@@ -147,7 +139,7 @@ pub fn init(client: Arc<Client>, cx: &mut AppContext) {
 pub enum CompletionProvider {
     OpenAi(OpenAiCompletionProvider),
     Anthropic(AnthropicCompletionProvider),
-    ZedDotDev(ZedDotDevCompletionProvider),
+    Cloud(CloudCompletionProvider),
     #[cfg(test)]
     Fake(FakeCompletionProvider),
 }
@@ -159,11 +151,30 @@ impl CompletionProvider {
         cx.global::<Self>()
     }
 
+    pub fn available_models(&self) -> Vec<LanguageModel> {
+        match self {
+            CompletionProvider::OpenAi(provider) => provider
+                .available_models()
+                .map(LanguageModel::OpenAi)
+                .collect(),
+            CompletionProvider::Anthropic(provider) => provider
+                .available_models()
+                .map(LanguageModel::Anthropic)
+                .collect(),
+            CompletionProvider::Cloud(provider) => provider
+                .available_models()
+                .map(LanguageModel::Cloud)
+                .collect(),
+            #[cfg(test)]
+            CompletionProvider::Fake(_) => unimplemented!(),
+        }
+    }
+
     pub fn settings_version(&self) -> usize {
         match self {
             CompletionProvider::OpenAi(provider) => provider.settings_version(),
             CompletionProvider::Anthropic(provider) => provider.settings_version(),
-            CompletionProvider::ZedDotDev(provider) => provider.settings_version(),
+            CompletionProvider::Cloud(provider) => provider.settings_version(),
             #[cfg(test)]
             CompletionProvider::Fake(_) => unimplemented!(),
         }
@@ -173,7 +184,7 @@ impl CompletionProvider {
         match self {
             CompletionProvider::OpenAi(provider) => provider.is_authenticated(),
             CompletionProvider::Anthropic(provider) => provider.is_authenticated(),
-            CompletionProvider::ZedDotDev(provider) => provider.is_authenticated(),
+            CompletionProvider::Cloud(provider) => provider.is_authenticated(),
             #[cfg(test)]
             CompletionProvider::Fake(_) => true,
         }
@@ -183,7 +194,7 @@ impl CompletionProvider {
         match self {
             CompletionProvider::OpenAi(provider) => provider.authenticate(cx),
             CompletionProvider::Anthropic(provider) => provider.authenticate(cx),
-            CompletionProvider::ZedDotDev(provider) => provider.authenticate(cx),
+            CompletionProvider::Cloud(provider) => provider.authenticate(cx),
             #[cfg(test)]
             CompletionProvider::Fake(_) => Task::ready(Ok(())),
         }
@@ -193,7 +204,7 @@ impl CompletionProvider {
         match self {
             CompletionProvider::OpenAi(provider) => provider.authentication_prompt(cx),
             CompletionProvider::Anthropic(provider) => provider.authentication_prompt(cx),
-            CompletionProvider::ZedDotDev(provider) => provider.authentication_prompt(cx),
+            CompletionProvider::Cloud(provider) => provider.authentication_prompt(cx),
             #[cfg(test)]
             CompletionProvider::Fake(_) => unimplemented!(),
         }
@@ -203,23 +214,19 @@ impl CompletionProvider {
         match self {
             CompletionProvider::OpenAi(provider) => provider.reset_credentials(cx),
             CompletionProvider::Anthropic(provider) => provider.reset_credentials(cx),
-            CompletionProvider::ZedDotDev(_) => Task::ready(Ok(())),
+            CompletionProvider::Cloud(_) => Task::ready(Ok(())),
             #[cfg(test)]
             CompletionProvider::Fake(_) => Task::ready(Ok(())),
         }
     }
 
-    pub fn default_model(&self) -> LanguageModel {
+    pub fn model(&self) -> LanguageModel {
         match self {
-            CompletionProvider::OpenAi(provider) => LanguageModel::OpenAi(provider.default_model()),
-            CompletionProvider::Anthropic(provider) => {
-                LanguageModel::Anthropic(provider.default_model())
-            }
-            CompletionProvider::ZedDotDev(provider) => {
-                LanguageModel::ZedDotDev(provider.default_model())
-            }
+            CompletionProvider::OpenAi(provider) => LanguageModel::OpenAi(provider.model()),
+            CompletionProvider::Anthropic(provider) => LanguageModel::Anthropic(provider.model()),
+            CompletionProvider::Cloud(provider) => LanguageModel::Cloud(provider.model()),
             #[cfg(test)]
-            CompletionProvider::Fake(_) => unimplemented!(),
+            CompletionProvider::Fake(_) => LanguageModel::default(),
         }
     }
 
@@ -231,7 +238,7 @@ impl CompletionProvider {
         match self {
             CompletionProvider::OpenAi(provider) => provider.count_tokens(request, cx),
             CompletionProvider::Anthropic(provider) => provider.count_tokens(request, cx),
-            CompletionProvider::ZedDotDev(provider) => provider.count_tokens(request, cx),
+            CompletionProvider::Cloud(provider) => provider.count_tokens(request, cx),
             #[cfg(test)]
             CompletionProvider::Fake(_) => futures::FutureExt::boxed(futures::future::ready(Ok(0))),
         }
@@ -244,7 +251,7 @@ impl CompletionProvider {
         match self {
             CompletionProvider::OpenAi(provider) => provider.complete(request),
             CompletionProvider::Anthropic(provider) => provider.complete(request),
-            CompletionProvider::ZedDotDev(provider) => provider.complete(request),
+            CompletionProvider::Cloud(provider) => provider.complete(request),
             #[cfg(test)]
             CompletionProvider::Fake(provider) => provider.complete(),
         }
