@@ -1,11 +1,13 @@
 //! Project-wide storage of the tasks available, capable of updating itself from the sources set.
 
 use std::{
+    borrow::Cow,
     cmp::{self, Reverse},
     path::{Path, PathBuf},
     sync::Arc,
 };
 
+use anyhow::Result;
 use collections::{btree_map, BTreeMap, VecDeque};
 use futures::{
     channel::mpsc::{unbounded, UnboundedSender},
@@ -13,13 +15,17 @@ use futures::{
 };
 use gpui::{AppContext, Context, Model, ModelContext, Task};
 use itertools::Itertools;
-use language::Language;
+use language::{ContextProvider, Language, Location};
 use task::{
-    static_source::StaticSource, ResolvedTask, TaskContext, TaskId, TaskTemplate, VariableName,
+    static_source::StaticSource, ResolvedTask, TaskContext, TaskId, TaskTemplate, TaskTemplates,
+    TaskVariables, VariableName,
 };
-use util::{post_inc, NumericPrefixWithSuffix};
+use text::{Point, ToPoint};
+use util::{post_inc, NumericPrefixWithSuffix, ResultExt};
 use worktree::WorktreeId;
 
+use crate::Project;
+
 /// Inventory tracks available tasks for a given project.
 pub struct Inventory {
     sources: Vec<SourceInInventory>,
@@ -42,11 +48,11 @@ pub enum TaskSourceKind {
     Worktree {
         id: WorktreeId,
         abs_path: PathBuf,
-        id_base: &'static str,
+        id_base: Cow<'static, str>,
     },
     /// ~/.config/zed/task.json - like global files with task definitions, applicable to any path
     AbsPath {
-        id_base: &'static str,
+        id_base: Cow<'static, str>,
         abs_path: PathBuf,
     },
     /// Languages-specific tasks coming from extensions.
@@ -186,13 +192,18 @@ impl Inventory {
     /// Deduplicates the tasks by their labels and splits the ordered list into two: used tasks and the rest, newly resolved tasks.
     pub fn used_and_current_resolved_tasks(
         &self,
-        language: Option<Arc<Language>>,
+        remote_templates_task: Option<Task<Result<Vec<(TaskSourceKind, TaskTemplate)>>>>,
         worktree: Option<WorktreeId>,
+        location: Option<Location>,
         task_context: &TaskContext,
-    ) -> (
+        cx: &AppContext,
+    ) -> Task<(
         Vec<(TaskSourceKind, ResolvedTask)>,
         Vec<(TaskSourceKind, ResolvedTask)>,
-    ) {
+    )> {
+        let language = location
+            .as_ref()
+            .and_then(|location| location.buffer.read(cx).language_at(location.range.start));
         let task_source_kind = language.as_ref().map(|language| TaskSourceKind::Language {
             name: language.name(),
         });
@@ -224,7 +235,7 @@ impl Inventory {
                 },
             );
         let not_used_score = post_inc(&mut lru_score);
-        let currently_resolved_tasks = self
+        let mut currently_resolved_tasks = self
             .sources
             .iter()
             .filter(|source| {
@@ -239,7 +250,7 @@ impl Inventory {
                     .into_iter()
                     .map(|task| (&source.kind, task))
             })
-            .chain(language_tasks)
+            .chain(language_tasks.filter(|_| remote_templates_task.is_none()))
             .filter_map(|(kind, task)| {
                 let id_base = kind.to_id_base();
                 Some((kind, task.resolve_task(&id_base, task_context)?))
@@ -254,71 +265,103 @@ impl Inventory {
             .collect::<Vec<_>>();
         let previously_spawned_tasks = task_usage
             .into_iter()
-            .map(|(_, (kind, task, lru_score))| (kind.clone(), task.clone(), lru_score));
+            .map(|(_, (kind, task, lru_score))| (kind.clone(), task.clone(), lru_score))
+            .collect::<Vec<_>>();
 
-        let mut tasks_by_label = BTreeMap::default();
-        tasks_by_label = previously_spawned_tasks.into_iter().fold(
-            tasks_by_label,
-            |mut tasks_by_label, (source, task, lru_score)| {
-                match tasks_by_label.entry((source, task.resolved_label.clone())) {
-                    btree_map::Entry::Occupied(mut o) => {
-                        let (_, previous_lru_score) = o.get();
-                        if previous_lru_score >= &lru_score {
-                            o.insert((task, lru_score));
+        let task_context = task_context.clone();
+        cx.spawn(move |_| async move {
+            let remote_templates = match remote_templates_task {
+                Some(task) => match task.await.log_err() {
+                    Some(remote_templates) => remote_templates,
+                    None => return (Vec::new(), Vec::new()),
+                },
+                None => Vec::new(),
+            };
+            let remote_tasks = remote_templates.into_iter().filter_map(|(kind, task)| {
+                let id_base = kind.to_id_base();
+                Some((
+                    kind,
+                    task.resolve_task(&id_base, &task_context)?,
+                    not_used_score,
+                ))
+            });
+            currently_resolved_tasks.extend(remote_tasks);
+
+            let mut tasks_by_label = BTreeMap::default();
+            tasks_by_label = previously_spawned_tasks.into_iter().fold(
+                tasks_by_label,
+                |mut tasks_by_label, (source, task, lru_score)| {
+                    match tasks_by_label.entry((source, task.resolved_label.clone())) {
+                        btree_map::Entry::Occupied(mut o) => {
+                            let (_, previous_lru_score) = o.get();
+                            if previous_lru_score >= &lru_score {
+                                o.insert((task, lru_score));
+                            }
+                        }
+                        btree_map::Entry::Vacant(v) => {
+                            v.insert((task, lru_score));
                         }
                     }
-                    btree_map::Entry::Vacant(v) => {
-                        v.insert((task, lru_score));
-                    }
-                }
-                tasks_by_label
-            },
-        );
-        tasks_by_label = currently_resolved_tasks.iter().fold(
-            tasks_by_label,
-            |mut tasks_by_label, (source, task, lru_score)| {
-                match tasks_by_label.entry((source.clone(), task.resolved_label.clone())) {
-                    btree_map::Entry::Occupied(mut o) => {
-                        let (previous_task, _) = o.get();
-                        let new_template = task.original_task();
-                        if new_template != previous_task.original_task() {
-                            o.insert((task.clone(), *lru_score));
+                    tasks_by_label
+                },
+            );
+            tasks_by_label = currently_resolved_tasks.iter().fold(
+                tasks_by_label,
+                |mut tasks_by_label, (source, task, lru_score)| {
+                    match tasks_by_label.entry((source.clone(), task.resolved_label.clone())) {
+                        btree_map::Entry::Occupied(mut o) => {
+                            let (previous_task, _) = o.get();
+                            let new_template = task.original_task();
+                            if new_template != previous_task.original_task() {
+                                o.insert((task.clone(), *lru_score));
+                            }
+                        }
+                        btree_map::Entry::Vacant(v) => {
+                            v.insert((task.clone(), *lru_score));
                         }
                     }
-                    btree_map::Entry::Vacant(v) => {
-                        v.insert((task.clone(), *lru_score));
-                    }
-                }
-                tasks_by_label
-            },
-        );
+                    tasks_by_label
+                },
+            );
 
-        let resolved = tasks_by_label
-            .into_iter()
-            .map(|((kind, _), (task, lru_score))| (kind, task, lru_score))
-            .sorted_by(task_lru_comparator)
-            .filter_map(|(kind, task, lru_score)| {
-                if lru_score < not_used_score {
-                    Some((kind, task))
-                } else {
-                    None
-                }
-            })
-            .collect();
-
-        (
-            resolved,
-            currently_resolved_tasks
+            let resolved = tasks_by_label
                 .into_iter()
-                .sorted_unstable_by(task_lru_comparator)
-                .map(|(kind, task, _)| (kind, task))
-                .collect(),
-        )
+                .map(|((kind, _), (task, lru_score))| (kind, task, lru_score))
+                .sorted_by(task_lru_comparator)
+                .filter_map(|(kind, task, lru_score)| {
+                    if lru_score < not_used_score {
+                        Some((kind, task))
+                    } else {
+                        None
+                    }
+                })
+                .collect::<Vec<_>>();
+
+            (
+                resolved,
+                currently_resolved_tasks
+                    .into_iter()
+                    .sorted_unstable_by(task_lru_comparator)
+                    .map(|(kind, task, _)| (kind, task))
+                    .collect(),
+            )
+        })
     }
 
-    /// Returns the last scheduled task, if any of the sources contains one with the matching id.
-    pub fn last_scheduled_task(&self) -> Option<(TaskSourceKind, ResolvedTask)> {
-        self.last_scheduled_tasks.back().cloned()
+    /// Returns the last scheduled task by task_id if provided.
+    /// Otherwise, returns the last scheduled task.
+    pub fn last_scheduled_task(
+        &self,
+        task_id: Option<&TaskId>,
+    ) -> Option<(TaskSourceKind, ResolvedTask)> {
+        if let Some(task_id) = task_id {
+            self.last_scheduled_tasks
+                .iter()
+                .find(|(_, task)| &task.id == task_id)
+                .cloned()
+        } else {
+            self.last_scheduled_tasks.back().cloned()
+        }
     }
 
     /// Registers task "usage" as being scheduled – to be used for LRU sorting when listing all tasks.
@@ -438,21 +481,6 @@ mod test_inventory {
         })
     }
 
-    pub(super) fn resolved_task_names(
-        inventory: &Model<Inventory>,
-        worktree: Option<WorktreeId>,
-        cx: &mut TestAppContext,
-    ) -> Vec<String> {
-        inventory.update(cx, |inventory, _| {
-            let (used, current) =
-                inventory.used_and_current_resolved_tasks(None, worktree, &TaskContext::default());
-            used.into_iter()
-                .chain(current)
-                .map(|(_, task)| task.original_task().label.clone())
-                .collect()
-        })
-    }
-
     pub(super) fn register_task_used(
         inventory: &Model<Inventory>,
         task_name: &str,
@@ -473,21 +501,146 @@ mod test_inventory {
         });
     }
 
-    pub(super) fn list_tasks(
+    pub(super) async fn list_tasks(
         inventory: &Model<Inventory>,
         worktree: Option<WorktreeId>,
         cx: &mut TestAppContext,
     ) -> Vec<(TaskSourceKind, String)> {
-        inventory.update(cx, |inventory, _| {
-            let (used, current) =
-                inventory.used_and_current_resolved_tasks(None, worktree, &TaskContext::default());
-            let mut all = used;
-            all.extend(current);
-            all.into_iter()
-                .map(|(source_kind, task)| (source_kind, task.resolved_label))
-                .sorted_by_key(|(kind, label)| (task_source_kind_preference(kind), label.clone()))
-                .collect()
-        })
+        let (used, current) = inventory
+            .update(cx, |inventory, cx| {
+                inventory.used_and_current_resolved_tasks(
+                    None,
+                    worktree,
+                    None,
+                    &TaskContext::default(),
+                    cx,
+                )
+            })
+            .await;
+        let mut all = used;
+        all.extend(current);
+        all.into_iter()
+            .map(|(source_kind, task)| (source_kind, task.resolved_label))
+            .sorted_by_key(|(kind, label)| (task_source_kind_preference(kind), label.clone()))
+            .collect()
+    }
+}
+
+/// A context provided that tries to provide values for all non-custom [`VariableName`] variants for a currently opened file.
+/// Applied as a base for every custom [`ContextProvider`] unless explicitly oped out.
+pub struct BasicContextProvider {
+    project: Model<Project>,
+}
+
+impl BasicContextProvider {
+    pub fn new(project: Model<Project>) -> Self {
+        Self { project }
+    }
+}
+
+impl ContextProvider for BasicContextProvider {
+    fn build_context(
+        &self,
+        _: &TaskVariables,
+        location: &Location,
+        cx: &mut AppContext,
+    ) -> Result<TaskVariables> {
+        let buffer = location.buffer.read(cx);
+        let buffer_snapshot = buffer.snapshot();
+        let symbols = buffer_snapshot.symbols_containing(location.range.start, None);
+        let symbol = symbols.unwrap_or_default().last().map(|symbol| {
+            let range = symbol
+                .name_ranges
+                .last()
+                .cloned()
+                .unwrap_or(0..symbol.text.len());
+            symbol.text[range].to_string()
+        });
+
+        let current_file = buffer
+            .file()
+            .and_then(|file| file.as_local())
+            .map(|file| file.abs_path(cx).to_string_lossy().to_string());
+        let Point { row, column } = location.range.start.to_point(&buffer_snapshot);
+        let row = row + 1;
+        let column = column + 1;
+        let selected_text = buffer
+            .chars_for_range(location.range.clone())
+            .collect::<String>();
+
+        let mut task_variables = TaskVariables::from_iter([
+            (VariableName::Row, row.to_string()),
+            (VariableName::Column, column.to_string()),
+        ]);
+
+        if let Some(symbol) = symbol {
+            task_variables.insert(VariableName::Symbol, symbol);
+        }
+        if !selected_text.trim().is_empty() {
+            task_variables.insert(VariableName::SelectedText, selected_text);
+        }
+        let worktree_abs_path = buffer
+            .file()
+            .map(|file| WorktreeId::from_usize(file.worktree_id()))
+            .and_then(|worktree_id| {
+                self.project
+                    .read(cx)
+                    .worktree_for_id(worktree_id, cx)
+                    .map(|worktree| worktree.read(cx).abs_path())
+            });
+        if let Some(worktree_path) = worktree_abs_path {
+            task_variables.insert(
+                VariableName::WorktreeRoot,
+                worktree_path.to_string_lossy().to_string(),
+            );
+            if let Some(full_path) = current_file.as_ref() {
+                let relative_path = pathdiff::diff_paths(full_path, worktree_path);
+                if let Some(relative_path) = relative_path {
+                    task_variables.insert(
+                        VariableName::RelativeFile,
+                        relative_path.to_string_lossy().into_owned(),
+                    );
+                }
+            }
+        }
+
+        if let Some(path_as_string) = current_file {
+            let path = Path::new(&path_as_string);
+            if let Some(filename) = path.file_name().and_then(|f| f.to_str()) {
+                task_variables.insert(VariableName::Filename, String::from(filename));
+            }
+
+            if let Some(stem) = path.file_stem().and_then(|s| s.to_str()) {
+                task_variables.insert(VariableName::Stem, stem.into());
+            }
+
+            if let Some(dirname) = path.parent().and_then(|s| s.to_str()) {
+                task_variables.insert(VariableName::Dirname, dirname.into());
+            }
+
+            task_variables.insert(VariableName::File, path_as_string);
+        }
+
+        Ok(task_variables)
+    }
+}
+
+/// A ContextProvider that doesn't provide any task variables on it's own, though it has some associated tasks.
+pub struct ContextProviderWithTasks {
+    templates: TaskTemplates,
+}
+
+impl ContextProviderWithTasks {
+    pub fn new(definitions: TaskTemplates) -> Self {
+        Self {
+            templates: definitions,
+        }
+    }
+}
+
+impl ContextProvider for ContextProviderWithTasks {
+    fn associated_tasks(&self) -> Option<TaskTemplates> {
+        Some(self.templates.clone())
     }
 }
 
@@ -499,9 +652,9 @@ mod tests {
     use super::*;
 
     #[gpui::test]
-    fn test_task_list_sorting(cx: &mut TestAppContext) {
+    async fn test_task_list_sorting(cx: &mut TestAppContext) {
         let inventory = cx.update(Inventory::new);
-        let initial_tasks = resolved_task_names(&inventory, None, cx);
+        let initial_tasks = resolved_task_names(&inventory, None, cx).await;
         assert!(
             initial_tasks.is_empty(),
             "No tasks expected for empty inventory, but got {initial_tasks:?}"
@@ -548,7 +701,7 @@ mod tests {
             &expected_initial_state,
         );
         assert_eq!(
-            resolved_task_names(&inventory, None, cx),
+            resolved_task_names(&inventory, None, cx).await,
             &expected_initial_state,
             "Tasks with equal amount of usages should be sorted alphanumerically"
         );
@@ -559,7 +712,7 @@ mod tests {
             &expected_initial_state,
         );
         assert_eq!(
-            resolved_task_names(&inventory, None, cx),
+            resolved_task_names(&inventory, None, cx).await,
             vec![
                 "2_task".to_string(),
                 "2_task".to_string(),
@@ -578,7 +731,7 @@ mod tests {
             &expected_initial_state,
         );
         assert_eq!(
-            resolved_task_names(&inventory, None, cx),
+            resolved_task_names(&inventory, None, cx).await,
             vec![
                 "3_task".to_string(),
                 "1_task".to_string(),
@@ -613,7 +766,7 @@ mod tests {
             &expected_updated_state,
         );
         assert_eq!(
-            resolved_task_names(&inventory, None, cx),
+            resolved_task_names(&inventory, None, cx).await,
             vec![
                 "3_task".to_string(),
                 "1_task".to_string(),
@@ -633,7 +786,7 @@ mod tests {
             &expected_updated_state,
         );
         assert_eq!(
-            resolved_task_names(&inventory, None, cx),
+            resolved_task_names(&inventory, None, cx).await,
             vec![
                 "11_hello".to_string(),
                 "3_task".to_string(),
@@ -650,7 +803,7 @@ mod tests {
     }
 
     #[gpui::test]
-    fn test_inventory_static_task_filters(cx: &mut TestAppContext) {
+    async fn test_inventory_static_task_filters(cx: &mut TestAppContext) {
         let inventory_with_statics = cx.update(Inventory::new);
         let common_name = "common_task_name";
         let path_1 = Path::new("path_1");
@@ -674,7 +827,7 @@ mod tests {
             );
             inventory.add_source(
                 TaskSourceKind::AbsPath {
-                    id_base: "test source",
+                    id_base: "test source".into(),
                     abs_path: path_1.to_path_buf(),
                 },
                 |tx, cx| {
@@ -688,7 +841,7 @@ mod tests {
             );
             inventory.add_source(
                 TaskSourceKind::AbsPath {
-                    id_base: "test source",
+                    id_base: "test source".into(),
                     abs_path: path_2.to_path_buf(),
                 },
                 |tx, cx| {
@@ -704,7 +857,7 @@ mod tests {
                 TaskSourceKind::Worktree {
                     id: worktree_1,
                     abs_path: worktree_path_1.to_path_buf(),
-                    id_base: "test_source",
+                    id_base: "test_source".into(),
                 },
                 |tx, cx| {
                     static_test_source(
@@ -719,7 +872,7 @@ mod tests {
                 TaskSourceKind::Worktree {
                     id: worktree_2,
                     abs_path: worktree_path_2.to_path_buf(),
-                    id_base: "test_source",
+                    id_base: "test_source".into(),
                 },
                 |tx, cx| {
                     static_test_source(
@@ -735,28 +888,28 @@ mod tests {
         let worktree_independent_tasks = vec![
             (
                 TaskSourceKind::AbsPath {
-                    id_base: "test source",
+                    id_base: "test source".into(),
                     abs_path: path_1.to_path_buf(),
                 },
                 "static_source_1".to_string(),
             ),
             (
                 TaskSourceKind::AbsPath {
-                    id_base: "test source",
+                    id_base: "test source".into(),
                     abs_path: path_1.to_path_buf(),
                 },
                 common_name.to_string(),
             ),
             (
                 TaskSourceKind::AbsPath {
-                    id_base: "test source",
+                    id_base: "test source".into(),
                     abs_path: path_2.to_path_buf(),
                 },
                 common_name.to_string(),
             ),
             (
                 TaskSourceKind::AbsPath {
-                    id_base: "test source",
+                    id_base: "test source".into(),
                     abs_path: path_2.to_path_buf(),
                 },
                 "static_source_2".to_string(),
@@ -769,7 +922,7 @@ mod tests {
                 TaskSourceKind::Worktree {
                     id: worktree_1,
                     abs_path: worktree_path_1.to_path_buf(),
-                    id_base: "test_source",
+                    id_base: "test_source".into(),
                 },
                 common_name.to_string(),
             ),
@@ -777,7 +930,7 @@ mod tests {
                 TaskSourceKind::Worktree {
                     id: worktree_1,
                     abs_path: worktree_path_1.to_path_buf(),
-                    id_base: "test_source",
+                    id_base: "test_source".into(),
                 },
                 "worktree_1".to_string(),
             ),
@@ -787,7 +940,7 @@ mod tests {
                 TaskSourceKind::Worktree {
                     id: worktree_2,
                     abs_path: worktree_path_2.to_path_buf(),
-                    id_base: "test_source",
+                    id_base: "test_source".into(),
                 },
                 common_name.to_string(),
             ),
@@ -795,7 +948,7 @@ mod tests {
                 TaskSourceKind::Worktree {
                     id: worktree_2,
                     abs_path: worktree_path_2.to_path_buf(),
-                    id_base: "test_source",
+                    id_base: "test_source".into(),
                 },
                 "worktree_2".to_string(),
             ),
@@ -810,9 +963,12 @@ mod tests {
             .sorted_by_key(|(kind, label)| (task_source_kind_preference(kind), label.clone()))
             .collect::<Vec<_>>();
 
-        assert_eq!(list_tasks(&inventory_with_statics, None, cx), all_tasks);
         assert_eq!(
-            list_tasks(&inventory_with_statics, Some(worktree_1), cx),
+            list_tasks(&inventory_with_statics, None, cx).await,
+            all_tasks
+        );
+        assert_eq!(
+            list_tasks(&inventory_with_statics, Some(worktree_1), cx).await,
             worktree_1_tasks
                 .iter()
                 .chain(worktree_independent_tasks.iter())
@@ -821,7 +977,7 @@ mod tests {
                 .collect::<Vec<_>>(),
         );
         assert_eq!(
-            list_tasks(&inventory_with_statics, Some(worktree_2), cx),
+            list_tasks(&inventory_with_statics, Some(worktree_2), cx).await,
             worktree_2_tasks
                 .iter()
                 .chain(worktree_independent_tasks.iter())
@@ -830,4 +986,26 @@ mod tests {
                 .collect::<Vec<_>>(),
         );
     }
+
+    pub(super) async fn resolved_task_names(
+        inventory: &Model<Inventory>,
+        worktree: Option<WorktreeId>,
+        cx: &mut TestAppContext,
+    ) -> Vec<String> {
+        let (used, current) = inventory
+            .update(cx, |inventory, cx| {
+                inventory.used_and_current_resolved_tasks(
+                    None,
+                    worktree,
+                    None,
+                    &TaskContext::default(),
+                    cx,
+                )
+            })
+            .await;
+        used.into_iter()
+            .chain(current)
+            .map(|(_, task)| task.original_task().label.clone())
+            .collect()
+    }
 }
