@@ -1212,6 +1212,7 @@ async fn test_create_directory_during_initial_scan(cx: &mut TestAppContext) {
         })
         .await
         .unwrap()
+        .to_included()
         .unwrap();
     assert!(entry.is_dir());
 
@@ -1268,6 +1269,7 @@ async fn test_create_dir_all_on_create_entry(cx: &mut TestAppContext) {
         })
         .await
         .unwrap()
+        .to_included()
         .unwrap();
     assert!(entry.is_file());
 
@@ -1310,6 +1312,7 @@ async fn test_create_dir_all_on_create_entry(cx: &mut TestAppContext) {
         })
         .await
         .unwrap()
+        .to_included()
         .unwrap();
     assert!(entry.is_file());
 
@@ -1329,6 +1332,7 @@ async fn test_create_dir_all_on_create_entry(cx: &mut TestAppContext) {
         })
         .await
         .unwrap()
+        .to_included()
         .unwrap();
     assert!(entry.is_file());
 
@@ -1346,6 +1350,7 @@ async fn test_create_dir_all_on_create_entry(cx: &mut TestAppContext) {
         })
         .await
         .unwrap()
+        .to_included()
         .unwrap();
     assert!(entry.is_file());
 
@@ -1673,7 +1678,7 @@ fn randomly_mutate_worktree(
             );
             let task = worktree.rename_entry(entry.id, new_path, cx);
             cx.background_executor().spawn(async move {
-                task.await?.unwrap();
+                task.await?.to_included().unwrap();
                 Ok(())
             })
         }
